<?php

namespace IPS\tutorials\api;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Tutorials Article API
 */
class _tutorials extends \IPS\Content\Api\ItemController
{
	/**
	 * Class
	 */
	protected $class = 'IPS\tutorials\Article';
	
	/**
	 * GET /tutorials/tutorials
	 * Get list of articles
	 *
	 * @apiparam	string	categories		Comma-delimited list of category IDs
	 * @apiparam	string	authors			Comma-delimited list of member IDs - if provided, only articles started by those members are returned
	 * @apiparam	int		locked			If 1, only articles which are locked are returned, if 0 only unlocked
	 * @apiparam	int		hidden			If 1, only articles which are hidden are returned, if 0 only not hidden
	 * @apiparam	int		pinned			If 1, only articles which are pinned are returned, if 0 only not pinned
	 * @apiparam	int		featured		If 1, only articles which are featured are returned, if 0 only not featured
	 * @apiparam	string	sortBy			What to sort by. Can be 'date' for creation date, 'title' or leave unspecified for ID
	 * @apiparam	string	sortDir			Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page			Page number
	 * @return		\IPS\Api\PaginatedResponse<\IPS\tutorials\Article>
	 */
	public function GETindex()
	{
		/* Where clause */
		$where = array();
				
		/* Return */
		return $this->_list( $where, 'categories' );
	}
	
	/**
	 * GET /tutorials/tutorials/{id}
	 * View information about a specific article
	 *
	 * @param		int		$id				ID Number
	 * @throws		ARTICLE/1	INVALID_ID		The article ID does not exist
	 * @throws		ARTICLE/1	INVALID_VERSION	The version ID does not exist
	 * @return		\IPS\tutorials\Article
	 */
	public function GETitem( $id )
	{
		try
		{
			$article = \IPS\tutorials\Article::load( $id );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'ARTICLE/1', 404 );
		}
	}
	
	/**
	 * POST /tutorials/tutorials
	 * Submit new article
	 *
	 * @reqapiparam	int					category		The ID number of the category the article should be created in
	 * @reqapiparam	int					author			The ID number of the member creating the article (0 for guest)
	 * @reqapiparam	string				title			The article name
	 * @reqapiparam	string				description		The description as HTML (e.g. "<p>This is an article.</p>")
	 * @apiparam	string				prefix			Prefix tag
	 * @apiparam	string				tags			Comma-separated list of tags (do not include prefix)
	 * @apiparam	datetime			date			The date/time that should be used for the article post date. If not provided, will use the current date/time
	 * @apiparam	string				ip_address		The IP address that should be stored for the article. If not provided, will use the IP address from the API request
	 * @apiparam	int					locked			1/0 indicating if the article should be locked
	 * @apiparam	int					hidden			0 = unhidden; 1 = hidden, pending moderator approval; -1 = hidden (as if hidden by a moderator)
	 * @apiparam	int					pinned			1/0 indicating if the article should be featured
	 * @apiparam	int					featured		1/0 indicating if the article should be featured
	 * @throws		ARTICLE/7			NO_CATEGEORY	The category ID does not exist
	 * @throws		ARTICLE/8			NO_AUTHOR		The author ID does not exist
	 * @throws		ARTICLE/9			NO_TITLE		No title was supplied
	 * @throws		ARTICLE/A			NO_DESC			No description was supplied
	 * @throws		ARTICLE/B			NO_ARTICLES		No articles were supplied
	 * @return		\IPS\tutorials\Article
	 */
	public function POSTindex()
	{
		/* Get category */
		try
		{
			$category = \IPS\tutorials\Category::load( \IPS\Request::i()->category );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'NO_CATEGEORY', 'ARTICLE/7', 400 );
		}
		
		/* Get author */
		if ( \IPS\Request::i()->author )
		{
			$author = \IPS\Member::load( \IPS\Request::i()->author );
			if ( !$author->member_id )
			{
				throw new \IPS\Api\Exception( 'NO_AUTHOR', 'ARTICLE/8', 400 );
			}
		}
		else
		{
			$author = new \IPS\Member;
		}
		
		/* Check we have a title and a description */
		if ( !\IPS\Request::i()->title )
		{
			throw new \IPS\Api\Exception( 'NO_TITLE', 'ARTICLE/9', 400 );
		}
		if ( !\IPS\Request::i()->description )
		{
			throw new \IPS\Api\Exception( 'NO_DESC', 'ARTICLE/A', 400 );
		}
		
		/* Return */
		$link->save();
		return new \IPS\Api\Response( 201, $link->apiOutput() );
	}
	
	/**
	 * POST /tutorials/tutorials/{id}
	 * Edit an article
	 *
	 * @apiparam	int					category		The ID number of the category the article should be created in
	 * @apiparam	int					author			The ID number of the member creating the article (0 for guest)
	 * @apiparam	string				title			The article name
	 * @apiparam	string				description		The description as HTML (e.g. "<p>This is an article.</p>")
	 * @apiparam	string				prefix			Prefix tag
	 * @apiparam	string				tags			Comma-separated list of tags (do not include prefix)
	 * @apiparam	datetime			date			The date/time that should be used for the article post date. If not provided, will use the current date/time
	 * @apiparam	string				ip_address		The IP address that should be stored for the article. If not provided, will use the IP address from the API request
	 * @apiparam	int					locked			1/0 indicating if the article should be locked
	 * @apiparam	int					hidden			0 = unhidden; 1 = hidden, pending moderator approval; -1 = hidden (as if hidden by a moderator)
	 * @apiparam	int					featured		1/0 indicating if the article should be featured
	 * @throws		ARTICLE/C			INVALID_ID		The article ID is invalid
	 * @throws		ARTICLE/D			NO_CATEGORY		The category ID does not exist
	 * @throws		ARTICLE/E			NO_AUTHOR		The author ID does not exist
	 * @return		\IPS\tutorials\Article
	 */
	public function POSTitem( $id )
	{
		try
		{
			$article = \IPS\tutorials\Article::load( $id );
			
			if ( isset( \IPS\Request::i()->category ) and \IPS\Request::i()->category != $article->category_id )
			{
				try
				{
					$newCategory = \IPS\tutorials\Category::load( \IPS\Request::i()->category );
					$article->move( $newCategory );
				}
				catch ( \OutOfRangeException $e )
				{
					throw new \IPS\Api\Exception( 'NO_CATEGORY', 'ARTICLE/D', 400 );
				}
			}
			
			/* New author */
			if ( isset( \IPS\Request::i()->author ) )
			{				
				try
				{
					$member = \IPS\Member::load( \IPS\Request::i()->author );
					if ( !$member->member_id )
					{
						throw new \OutOfRangeException;
					}
					
					$article->changeAuthor( $member );
				}
				catch ( \OutOfRangeException $e )
				{
					throw new \IPS\Api\Exception( 'NO_AUTHOR', 'ARTICLE/E', 400 );
				}
			}
						
			/* Everything else */
			$this->_createOrUpdate( $article );
			
			/* Save and return */
			$article->save();
			return new \IPS\Api\Response( 200, $article->apiOutput() );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'ARTICLE/C', 404 );
		}
	}
	
	/**
	 * Create or update an article
	 *
	 * @param	\IPS\Content\Item	$item	The item
	 * @return	\IPS\Content\Item
	 */
	protected function _createOrUpdate( \IPS\Content\Item $item )
	{
		/* Description */
		if ( isset( \IPS\Request::i()->description ) )
		{
			$item->desc = \IPS\Request::i()->description;
		}
		
		/* Pass up */
		return parent::_createOrUpdate( $item );
	}

	
	/**
	 * GET /tutorials/tutorials/{id}/comments
	 * Get comments on an article
	 *
	 * @param		int		$id			ID Number
	 * @apiparam	int		hidden		If 1, only comments which are hidden are returned, if 0 only not hidden
	 * @apiparam	string	sortDir		Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page		Page number
	 * @throws		ARTICLE/2	INVALID_ID	The article ID does not exist
	 * @return		\IPS\Api\PaginatedResponse<\IPS\tutorials\Article\Comment>
	 */
	public function GETitem_comments( $id )
	{
		try
		{
			return $this->_comments( $id, '\IPS\tutorials\Article\Comment' );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'ARTICLE/2', 404 );
		}
	}
	
	/**
	 * GET /tutorials/tutorials/{id}/reviews
	 * Get reviews on an article
	 *
	 * @param		int		$id			ID Number
	 * @apiparam	int		hidden		If 1, only comments which are hidden are returned, if 0 only not hidden
	 * @apiparam	string	sortDir		Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page		Page number
	 * @throws		ARTICLE/3	INVALID_ID	The article ID does not exist
	 * @return		\IPS\Api\PaginatedResponse<\IPS\tutorials\Article\Review>
	 */
	public function GETitem_reviews( $id )
	{
		try
		{
			return $this->_comments( $id, '\IPS\tutorials\Article\Review' );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'ARTICLE/3', 404 );
		}
	}
	
	/**
	 * DELETE /tutorials/tutorials/{id}
	 * Delete an article
	 *
	 * @param		int		$id			ID Number
	 * @throws		ARTICLE/5	INVALID_ID	The article ID does not exist
	 * @return		void
	 */
	public function DELETEitem( $id )
	{
		try
		{
			\IPS\tutorials\Article::load( $id )->delete();
			
			return new \IPS\Api\Response( 200, NULL );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'ARTICLE/5', 404 );
		}
	}
}